package org.uoc.apptest.config;

import org.uoc.winter.anotacions.ContextAttr;

/*
 * Aquesta classe la farem servir de configuracio, assignant camps amb valors constants
 * perque el framework les pugui recollir
 */
public class Contexte
{
    /*
     *  aquest atribut especifica on anirem a buscar els controladors i substitueix al configuracio
     *  necessaria a servlet-context.xml; si n'hi ha més d'un paquet a explorar, s'ha de separar amb ';'
     *  ex. String scan="org.uoc.xxx;org.uoc.yyy";
     *  
     *  SCAN=classe;classe indica la classe o classes on hem de buscar els controladors
     */
	
	@ContextAttr(group="CONTROLLERS",tag="SCAN")
    static final String controllers ="org.uoc.apptest.mvc;"+
	                                 "org.uoc.apptest.main";
	
    /*
     * aquests atribut ens ajudaran a establir els camins per els diferents tipus de vistes, si es necessari
     * Podem posar mes d'un cami per cada vista, separat per ';'
     */
	
	@ContextAttr(group="VIEWS",tag="ANY")
    static final String views_any  ="/WEB-INF/views/apptest";
	
	@ContextAttr(group="VIEWS",tag="JSP")
    static final String views_jsp  ="/WEB-INF/views/apptest/jsp";
	
	@ContextAttr(group="VIEWS",tag="HTML")
    static final String views_html ="/WEB-INF/views/apptest/html;"+
	                                "/WEB-INF/views/apptest/error-pages";
 
	/*
	 * Aquests atributs especifiquem la pagina web a la que es refereixen les vistes
	 * Utilitzats en cas que el controlador reotrni un nom de vista en comptes d'una fitxer
	 * el valor especifica el nom de la vista i el fitxer html relacionat, separats per el simbol (->)
	 * Es poden especificar diferents elements separats per (;)
	 * 
	 * Tambe podem associar un nom de vista amb un controlador determinat. La vista es correspondra
	 * amb el metode indicat del controlador. Si no s'especifica un nom de mètode, es buscarà un metode amb
	 * el mateix nom de la vista referida. la sintaxis es:
	 *    nom_de_vista->Controlador:metode
	 */
	@ContextAttr(group="VIEWS",tag="NAMES")
	static final String view_names ="error->error.html;" +
			                        "newprofile->org.uoc.apptest.mvc.ProfileController:newprofile;" +
			                        "addprofile->org.uoc.apptest.mvc.ProfileController:addprofile;" +
			                        "principal->org.uoc.apptest.main.MainController";
	
	/*
	 * Aquest atribut indica les classes de validacio de formularis
	 *
	@ContextAttr(group="VALIDATION",tag="CLASS")
    static final String validation ="org.uoc.apptest.Validate";
*/
	/* interceptors d'events, que s'excutaran abans i despres dels controladors
	 * com a valor de parametre, es passa el nom de la classe i el metode, separat per dos punts (:) */
    @ContextAttr(group="INTERCEPTOR",tag="PRE")
    static final String interceptor_pre ="org.uoc.apptest.Interceptor:preexecute";

    @ContextAttr(group="INTERCEPTOR",tag="POST")
    static final String interceptor_post ="org.uoc.apptest.Interceptor:postexecute";

}