package org.uoc.winter.listeners;

import org.uoc.winter.utils.Configuracio;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.servlet.ServletContext;

import org.apache.log4j.Logger;


/*
 * Aquesta classe s'encarrega de carregar i processar el fitxer de preferencies especificat
 * a log4jXML. Esta basada en un listener de log4j pero per evitar la necessitat de declarar-ho
 * desde web.xml, ho iniciarem automaticament des del dispatcher servlet quan l'usuari especifiqui
 * un fitxer de configuracio a traves del tag LOG4J
 * Val a dir que JBOSS utilitza la seva propia configuracio de LOG4J
 */
public class ContextListener implements ServletContextListener
{
	private final Logger log = Logger.getRootLogger();
	private static final String N_SEP=";";
	
    public void contextDestroyed(ServletContextEvent sce)
    {
    	String loghead="(contextListener) contextDestroyed> ";
    	
    	log.info("begin");
    	
    	ServletContext ctx = sce.getServletContext();
        String name = ctx.getServletContextName();
        
        log.info(loghead+"aturat: " + (name == null ? "" : name ));
        
        log.info(loghead+"begin");
    }

    public void contextInitialized(ServletContextEvent sce)
    {
    	String loghead="(contextListener) contextInitialized> ";
        
    	log.info(loghead+"begin");
    	
    	ServletContext ctx = sce.getServletContext();
    	
    	if (ctx==null) return;
    	
        String name = ctx.getServletContextName();
        
        String config=ctx.getInitParameter("contextConfigLocation");
		
        /*
		 * llegeix i parseja el fitxer de configuracio
		 */
        Configuracio conf = new Configuracio(config,N_SEP);
		
		/*
		 * En aquest punt tenim els parametres de configuracio de l'usuari carregats
		 */
			
		/*
		 * convertim els tags de configuracio en atributs que estaran disponibles a DispatcherServlet
		 */
		 Set<Map.Entry<String, String>> set;
			
		 set = conf.get_tags().entrySet();
		 Iterator<Map.Entry<String, String>> it;
		 
		 for (it=set.iterator();it.hasNext();) { 
		 	Map.Entry<String,String> entry = it.next(); 
			
		 	String key = entry.getKey();
			String val = entry.getValue();
			
			log.info(loghead+key+"="+val);
				
			/*
			 * publiquem el valor de l'atribut al context
			 */
			ctx.setAttribute(key,val);
		}
    
        log.info(loghead+"iniciat: "+(name == null ? "" : name ));
        
        log.info(loghead+"end");
  
    }
}