package org.uoc.winter.listeners;

import javax.servlet.ServletContextListener;
import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;

import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;

/*
 * Aquesta classe s'encarrega de carregar i processar el fitxer de preferencies especificat
 * a log4jXML. Esta basada en un listener de log4j pero per evitar la necessitat de declarar-ho
 * desde web.xml, ho iniciarem automaticament des del dispatcher servlet quan l'usuari especifiqui
 * un fitxer de configuracio a traves del tag LOG4J
 * Val a dir que JBOSS utilitza la seva propia configuracio de LOG4J
 */
public class Log4jConfigListener  implements ServletContextListener
{
	private final Logger log = Logger.getRootLogger();
	 
    public void contextDestroyed(ServletContextEvent sce)
    {
    	String loghead="(Log4jConfigListener) contextDestroyed> ";
    	
    	ServletContext ctx = sce.getServletContext();
        String name = ctx.getServletContextName();
        
        // hauriem de tancar el proces de log
        log.info(loghead+"aturat: " + (name == null ? "" : name ));
    }
 

    public void contextInitialized(ServletContextEvent sce)
    {
    	String loghead="(Log4jConfigListener) contextInitialized> ";
    	
    	ServletContext ctx = sce.getServletContext();
        String name = ctx.getServletContextName();
        
        String realPath = ctx.getRealPath("/");
        String fileSep = System.getProperty("file.separator");
        
        if (realPath != null && (! realPath.endsWith(fileSep))){
            realPath = realPath + fileSep;}
        
        /*
         * si falla la configuracio (no s'aconsegueix trobar el fitxer) sortim i 
         * indiquem a DispatcherServlet la situacio
         */
        try {
           // llegim el fitxer de configuracio
           DOMConfigurator.configure(realPath+ctx.getInitParameter("log4jXML"));
           
        }
        catch (Exception e) {
        	e.printStackTrace();
        	return;
        }
        
        // iniciem el proces de log

        log.info(loghead+"iniciat: " + (name == null ? "" : name ));
        
        log.fatal(loghead+"Això és un log FATAL");
        log.error(loghead+"Això és un log ERROR");
        log.warn (loghead+"Això és un log WARN");
        log.info (loghead+"Això és un log INFO");
        log.debug(loghead+"Això és un log DEBUG");

    }
}