package org.uoc.winter.mvc;

import org.uoc.winter.enumeracions.HttpMethod;
import org.uoc.winter.renderers.ContentView;
import org.uoc.winter.renderers.UrlView;
import org.uoc.winter.utils.Configuracio;
import org.uoc.winter.utils.UtilJavassist;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;

import java.util.Enumeration;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Constructor;

import org.apache.log4j.Logger;


public class DispatcherServlet extends HttpServlet {
	public static final long serialVersionUID = 1L;
	private final Logger log = Logger.getRootLogger();
	private ServletContext SrvltCntx;
	private Mapes mapes = new Mapes();
	private ViewResolver viewresolver = new ViewResolver();
	
	private static final String N_SEP=";";

	public void init(ServletConfig SrvltConf) throws ServletException {
		
        String loghead="init> ";
		
		log.info(loghead+"begin");
		
		// Inicialitzacio de HttpServlet
		super.init(SrvltConf);
		
		/*
		 * obtenir el contexte desde la configuracio
		 */
		SrvltCntx = SrvltConf.getServletContext();
		
		log.info(loghead+"list servlet parameters");
		
		/*
		 * Mostrem en el log els parametres que han passat desde web.xml
		 */
		Enumeration<String> enumeration = SrvltCntx.getInitParameterNames();
		while(enumeration.hasMoreElements()) {
		  log.info(loghead+"initparameter="+enumeration.nextElement());
		}
		  
		log.info(loghead+"list servlet parameters done");

		log.info(loghead+"list servlet attributes");
		
		/*
		 * mostrem els atributs assignats des del ContextListener
		 */
		Enumeration<String> attrs = SrvltCntx.getAttributeNames();
		while (attrs.hasMoreElements()) {
			String attr = attrs.nextElement();
            String [] parts_attr;
            Configuracio.Grups group;
            Configuracio.Tags  tag;
            
			if (!Configuracio.AtributValid(attr)) continue;
			
			Object value = SrvltCntx.getAttribute(attr);
			
			// els atributs estan formats per group@tag
			parts_attr = attr.split("@");
			
			group = Configuracio.grup(parts_attr[0]);
			tag   = Configuracio.tag(parts_attr[1]);

			if (value instanceof String) {
			   String val = (String)value;

			   log.info(loghead+attr+"="+val);
			   
			   /*
			    * processem les anotacions de un nom de classe pasada per parametre (fully-qualified=amb el nom de paquet inclos)
			    */
			  		
			   if (group==Configuracio.Grups.CONTROLLERS && tag==Configuracio.Tags.SCAN) {
			      log.info(loghead+">> CONTROLLERS (SCAN)  BEGIN <<");
			      
			      /*
			       * inicialitzem Javaassist i buscant les classes que pertanyen als paquets
			       */
			      UtilJavassist util = new UtilJavassist(SrvltCntx,val,N_SEP);
			      util.ProcessarControllers(mapes);
			      
			      log.info(loghead+"created "+mapes.size()+" maps");
			      mapes.log();
			      
			      log.info(loghead+">> CONTROLLERS (SCAN)   END  <<");
			   }
			   else if (group==Configuracio.Grups.VIEWS && tag==Configuracio.Tags.ANY) {
				   log.info(loghead+">>     VIEWS  (ANY)   BEGIN <<");
				   
				   String [] paths = val.split(N_SEP);
				   viewresolver.add_anys(paths);
				   viewresolver.log_anys();
				      
				   log.info(loghead+">>     VIEWS  (ANY)    END  <<");
			   }
			   else if (group==Configuracio.Grups.VIEWS && tag==Configuracio.Tags.JSP) {
				   log.info(loghead+">>     VIEWS  (JSP)   BEGIN <<");
				   
				   String [] paths = val.split(N_SEP);
				   viewresolver.add_jsps(paths);
				   viewresolver.log_jsps();
				      
				   log.info(loghead+">>     VIEWS  (JSP)    END  <<");
			   }
			   else if (group==Configuracio.Grups.VIEWS && tag==Configuracio.Tags.HTML) {
				   log.info(loghead+">>     VIEWS  (HTML)  BEGIN <<");
				   
				   String [] paths = val.split(N_SEP);
				   viewresolver.add_htmls(paths);
				   viewresolver.log_htmls();
				   
				   log.info(loghead+">>     VIEWS  (HTML)   END  <<");
			   }
			   else if (group==Configuracio.Grups.VIEWS && tag==Configuracio.Tags.NAMES) {
				   log.info(loghead+">>     VIEWS  (NAMES)  BEGIN <<");
				   
				   String [] names = val.split(N_SEP);
				   viewresolver.add_names(names);
				   viewresolver.log_names();
				   
				   log.info(loghead+">>     VIEWS  (NAMES)   END  <<");
			   }
			}
		}
		
		log.info(loghead+"list servlet attributes done");

		log.info(loghead+"end");
	}
	
	public void destroy() {
		String loghead="destroy> ";
		
		log.info(loghead+"begin");
	    
	    log.info(loghead+"end");
	}
	
    public void doGet(HttpServletRequest req, HttpServletResponse answ) throws ServletException, IOException {
    	respon(HttpMethod.GET,req, answ);
    }
    
   	public void doPost(HttpServletRequest req, HttpServletResponse answ) throws ServletException, IOException {
        respon(HttpMethod.POST,req, answ);
   	}
   	
   	public void respon(HttpMethod metode,HttpServletRequest req, HttpServletResponse answ) throws ServletException, IOException {
   		String loghead="respon> ";
   		
   		log.info(loghead+"begin");
   		log.info(loghead+"method="+metode.toString());

        String contextpath  = req.getContextPath();
        String servletpath  = req.getServletPath();
        String infopath     = req.getPathInfo();
        String path         = ( infopath != null ? infopath : servletpath ) ;

        log.info(loghead+"contextpath="+contextpath);
        log.info(loghead+"servletpath="+servletpath);
        log.info(loghead+"infopath="+infopath);
        
        // primer busquem la pagina en algun dels camis especificats
        String view = viewresolver.search(SrvltCntx,path);

        // si hem trobat una vista valida, la mostrem
        if (view!=null) {
        	
        	String classname=viewresolver.get_renderclass();
        	log.info(loghead+"pagina '"+view+"' trobada. renderer="+classname);
        	
        	try {
        	    Class<?> renderclass = Class.forName(classname);
        	    Constructor<?> constructor = renderclass.getConstructor(String.class);
        	    
        	    UrlView rv = (UrlView)constructor.newInstance(new Object [] { view } );
        	    rv.render(req, answ);
        	}
        	catch (Exception e) {
        		e.printStackTrace();
        	}   	
        	
        }
        else {
           // si no, mostrem la nostra pagina d'error
           String html=
                   "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">"+
                   "<HTML>"+
                   " <HEAD><TITLE>Winter MVC: Page Error!</TITLE></HEAD>"+
                   " <BODY>"+
                   "<h1>Winter MVC</h1>"+
                   "<h2><font color='#aa0000'>Error!<br>Your page '<i>"+path+"</i>' could not be reached</font></h2>"+
                   " </BODY>"+
                   "</HTML>";
                   
        	String classname="org.uoc.winter.renderers.ContentView";
        	
        	try {
        	    Class<?> renderclass = Class.forName(classname);
        	    Constructor<?> constructor = renderclass.getConstructor(String.class,String.class);
        	    
        	    ContentView rv = (ContentView)constructor.newInstance(new Object [] { "text/html",html } );
        	    rv.render(req, answ);
        	}
        	catch (Exception e) {
        		e.printStackTrace();
        	}
        }
        
        // if (infopath.equals("aixo fa un error")) { log.info(loghead+"error"); }
     /*
        if (infopath==null) view="error.html";
        else {        
        	if      (path.equals("action1"))   view = action1();
            else if (path.equals("action2"))   view = action2();
        }

        log.info(loghead+"view="+view);

        if (view!=null) {
            
        	RequestDispatcher rd = req.getRequestDispatcher(view);
            
            if (rd!=null) { 
                log.info(loghead+"end . forwarding to "+view);
            	rd.forward(req,answ);
            }
            else {
               log.error(loghead+"Error: Unable to resolve view: "+view);
            }
        }
       */ 
        log.info(loghead+"end");
    }
/*
   	protected void doDispatch(HttpServletRequest request, HttpServletResponse response) throws Exception {
   	    HttpServletRequest processedRequest = request;
   	    HandlerExecutionChain mappedHandler = null;
   	    int interceptorIndex = -1;
   	 
   	    try {
   	        ModelAndView mv;
   	        boolean errorView = false;
   	 
   	        try {
   	            processedRequest = checkMultipart(request);
   	 
   	            // Determine handler for the current request.
   	            mappedHandler = getHandler(processedRequest, false);
   	            if (mappedHandler == null || mappedHandler.getHandler() == null) {
   	                noHandlerFound(processedRequest, response);
   	                return;
   	            }
   	 
   	            // Determine handler adapter for the current request.
   	            HandlerAdapter ha = getHandlerAdapter(mappedHandler.getHandler());
   	 
   	                       // Process last-modified header, if supported by the handler.
   	            String method = request.getMethod();
   	            boolean isGet = "GET".equals(method);
   	            if (isGet || "HEAD".equals(method)) {
   	                long lastModified = ha.getLastModified(request, mappedHandler.getHandler());
   	                if (logger.isDebugEnabled()) {
   	                    String requestUri = urlPathHelper.getRequestUri(request);
   	                    logger.debug("Last-Modified value for [" + requestUri + "] is: " + lastModified);
   	                }
   	                if (new ServletWebRequest(request, response).checkNotModified(lastModified) && isGet) {
   	                    return;
   	                }
   	            }
   	 
   	            // Apply preHandle methods of registered interceptors.
   	            HandlerInterceptor[] interceptors = mappedHandler.getInterceptors();
   	            if (interceptors != null) {
   	                for (int i = 0; i < interceptors.length; i++) {                  HandlerInterceptor interceptor = interceptors[i];                   if (!interceptor.preHandle(processedRequest, response, mappedHandler.getHandler())) {                       triggerAfterCompletion(mappedHandler, interceptorIndex, processedRequest, response, null);                      return;                     }                   interceptorIndex = i;               }           }           // Actually invoke the handler.             mv = ha.handle(processedRequest, response, mappedHandler.getHandler());             // Do we need view name translation?            if (mv != null && !mv.hasView()) {              mv.setViewName(getDefaultViewName(request));            }           // Apply postHandle methods of registered interceptors.             if (interceptors != null) {                 for (int i = interceptors.length - 1; i >= 0; i--) {
   	                    HandlerInterceptor interceptor = interceptors[i];
   	                    interceptor.postHandle(processedRequest, response, mappedHandler.getHandler(), mv);
   	                }
   	            }
   	        }
   	        catch (ModelAndViewDefiningException ex) {
   	            logger.debug("ModelAndViewDefiningException encountered", ex);
   	            mv = ex.getModelAndView();
   	        }
   	        catch (Exception ex) {
   	            Object handler = (mappedHandler != null ? mappedHandler.getHandler() : null);
   	            mv = processHandlerException(processedRequest, response, handler, ex);
   	            errorView = (mv != null);
   	        }
   	 
   	        // Did the handler return a view to render?
   	        if (mv != null && !mv.wasCleared()) {
   	            render(mv, processedRequest, response);
   	            if (errorView) {
   	                WebUtils.clearErrorRequestAttributes(request);
   	            }
   	        }
   	        else {
   	            if (logger.isDebugEnabled()) {
   	                logger.debug("Null ModelAndView returned to DispatcherServlet with name '" + getServletName() +
   	                        "': assuming HandlerAdapter completed request handling");
   	            }
   	        }
   	 
   	        // Trigger after-completion for successful outcome.
   	        triggerAfterCompletion(mappedHandler, interceptorIndex, processedRequest, response, null);
   	    }
   	 
   	    catch (Exception ex) {
   	        // Trigger after-completion for thrown exception.
   	        triggerAfterCompletion(mappedHandler, interceptorIndex, processedRequest, response, ex);
   	        throw ex;
   	    }
   	    catch (Error err) {
   	        ServletException ex = new NestedServletException("Handler processing failed", err);
   	        // Trigger after-completion for thrown exception.
   	        triggerAfterCompletion(mappedHandler, interceptorIndex, processedRequest, response, ex);
   	        throw ex;
   	    }
   	 
   	    finally {
   	        // Clean up any resources used by a multipart request.
   	        if (processedRequest != request) {
   	            cleanupMultipart(processedRequest);
   	        }
   	    }
   	}
   	*/
    public String action1() { return "view1.jsp"; }
    public String action2() { return "view2.jsp"; }         
}

