package org.uoc.winter.mvc;

import java.io.File;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;

import javax.servlet.ServletContext;

import org.apache.log4j.Logger;

public class ViewResolver {
	private ArrayList<String> JspPaths;
	private ArrayList<String> HtmlPaths;
	private ArrayList<String> AnyPaths;
	private ArrayList<String> Names;
	private final Logger log = Logger.getRootLogger();
	private String renderclass;
	private enum ViewType {

        HTML, JSP, TXT, CONTROLLER

    }
	
	private HashMap<ViewType,String> ViewClasses;
	
	public ViewResolver() {
		JspPaths  = new ArrayList<String>();
		HtmlPaths = new ArrayList<String>();
		AnyPaths  = new ArrayList<String>();
		Names     = new ArrayList<String>();
	    renderclass="";
	    ViewClasses = new HashMap<ViewType,String>();
	    
	    /* inicialitzem els valors de renderització per cadascun dels tipus de vistes */
	    ViewClasses.put(ViewType.HTML,"org.uoc.winter.renderers.UrlView");
	    ViewClasses.put(ViewType.JSP,"org.uoc.winter.renderers.UrlView");
	    ViewClasses.put(ViewType.TXT,"org.uoc.winter.renderers.ContentView");
	    ViewClasses.put(ViewType.CONTROLLER,"org.uoc.winter.renderers.ControllerView");
	}
	
	public void add_jsp(String path) {
		JspPaths.add(path);
	}
	
	public void add_html(String path) {
		HtmlPaths.add(path);
	}
	
	public void add_any(String path) {
		AnyPaths.add(path);
	}
	
	public void add_name(String name) {
		Names.add(name);
	}
	
	public void add_jsps(String [] paths) {
		Collections.addAll(JspPaths, paths);
	}
	
	public void add_htmls(String [] paths) {
		Collections.addAll(HtmlPaths, paths);
	}
	
	public void add_anys(String [] paths) {
		Collections.addAll(AnyPaths, paths);
	}
	
	public void add_names(String [] names) {
		Collections.addAll(Names, names);
	}
	
	public void   set_renderclass(String val) { renderclass=val; }
	public String get_renderclass()           { return renderclass; }
	
	public void log_jsps() {
	   String loghead="ViewResolver (JSP)> ";
	   
	   log.info(loghead+JspPaths.size()+" JspPaths");
	   for (Iterator<String> it = JspPaths.iterator(); it.hasNext();) {
		  String val = (String)it.next();
		  log.info(loghead+"Path: "+val);
	   }
	}
	
	public void log_htmls() {
	   String loghead="ViewResolver (HTML)> ";
		   
	   log.info(loghead+HtmlPaths.size()+" HtmlPaths");
	   for (Iterator<String> it = HtmlPaths.iterator(); it.hasNext();) {
	      String val = (String)it.next();
	      log.info(loghead+"Path: "+val);
	   }
    }
	
	public void log_anys() {
	   String loghead="ViewResolver (ANY)> ";
			   
	   log.info(loghead+AnyPaths.size()+" AnyPaths");
	   for (Iterator<String> it = HtmlPaths.iterator(); it.hasNext();) {
	      String val = (String)it.next();
	      log.info(loghead+"Path: "+val);
	   }
	}
	
	public void log_names() {
	   String loghead="ViewResolver (NAME)> ";
			   
	   log.info(loghead+Names.size()+" Names");
	   for (Iterator<String> it = Names.iterator(); it.hasNext();) {
		  String val = (String)it.next();
		  log.info(loghead+"Name: "+val);
	   }
	}
	
	private boolean check_file(String path) {
		 String loghead="check_file> ";
		 
		 File f = new File(path);
		 log.info(loghead+f.getAbsolutePath()+" exists:"+f.exists());
		 return f.exists();
	}
	
	private ViewType viewtype(String UppPath) {
		if (UppPath.endsWith(".HTML") || UppPath.endsWith(".HTM")) return ViewType.HTML;
		if (UppPath.endsWith(".JSP") || UppPath.endsWith(".JSTL")) return ViewType.JSP;
		
		return ViewType.CONTROLLER;
	}
	
	private String lookup(ServletContext ctx,ArrayList<String> Paths,String path) {
		String loghead="lookup> ";
		
		log.info(loghead+"begin");
		
	       for (Iterator<String> it = Paths.iterator(); it.hasNext();) {
	 		   String val = (String)it.next();
	    	   String cami = val;
	    		  
	    	   log.info(loghead+"path="+path+" Path: "+val);
	    		  
	    	   // afegim (/) al cami si no existeix
	    	   if (! cami.endsWith("/") ) cami+="/";
	    	   cami+=path;
	    		  
	    	   // hem de mirar si el fitxer existeix
	      	   if (check_file(ctx.getRealPath(cami))) { 
	    	   log.info(loghead+"fitxer trobat");
	    		  return cami;
	          }   
	       }
	    
	       log.info(loghead+"end");
	       
	       return "";
	}
	
	/* aquest metode tradueix un cami passat per parametre en un fitxer
	 * valid per ser renderitzat
	 */
	public String search(ServletContext ctx,String path) {
	   String loghead="ViewResolver> ";
	   
	   log.info(loghead+"begin");
	   log.info(loghead+"path="+path);
	   
       log.info(loghead+"looking as view name ...");
       
       // retirem le possible "/" al principi
       if (path.startsWith("/")) path = path.substring(1);
       
       for (Iterator<String> it = Names.iterator(); it.hasNext();) {
		  String val = (String)it.next();
		     
		  String [] parts = val.split("->");
		  if (parts.length==2) { // que estigui ben format ...
			  log.info(loghead+"path="+path+" Name: "+val+" parts[0]="+parts[0]+" parts[1]="+parts[1]);
		   	  
			  if (parts[0].equalsIgnoreCase(path)) {
		   		  path = parts[1];
		   	      break;
		   	  }
	      }
	   }
     	 
       /* retirem le possible "/" al principi (un altre cop, ja que pots haver
        * estat modificat per la iteracio anterior
        */
       if (path.startsWith("/")) path = path.substring(1);
       
       String UppPath = path.toUpperCase();
       
       if (viewtype(UppPath)==ViewType.HTML) {
       	   log.info(loghead+"looking html ...");
       	   
       	   String cami = lookup(ctx,AnyPaths,path);
       	   if (cami.isEmpty()) cami = lookup(ctx,HtmlPaths,path);
       	   
    	   if (! cami.isEmpty()) {
    		   log.info(loghead+"trobat cami="+cami);
    		   set_renderclass(ViewClasses.get(viewtype(UppPath)));
    		   return cami;
    	   }
       }
       else if (viewtype(UppPath)==ViewType.JSP) {
       	   log.info(loghead+"looking jsp ...");
       	   
       	   String cami = lookup(ctx,AnyPaths,path);
       	   if (cami.isEmpty()) cami = lookup(ctx,JspPaths,path);
       	   
    	   if (! cami.isEmpty()) {
    		   log.info(loghead+"trobat cami="+cami);
    		   set_renderclass(ViewClasses.get(viewtype(UppPath)));
    		   return cami;
    	   }
       }
       else if (viewtype(UppPath)==ViewType.CONTROLLER){
       	   // en aquest cas, s'especifica el nom del controlador i el mètode
           log.info(loghead+"looking controller ...");
           
           if (! path.isEmpty()) {
        	  log.info(loghead+"trobat controller="+path);
        	   
              set_renderclass(ViewClasses.get(viewtype(UppPath)));
              return path;
           }
       }
       
              
	   log.info(loghead+"end");
	   
	   return null;
	}
}