package org.uoc.winter.utils;

import org.uoc.winter.mvc.Mapes;
import org.uoc.winter.mvc.Mapa;
import org.uoc.winter.anotacions.ContextAttr;
import org.uoc.winter.anotacions.Controller;
import org.uoc.winter.anotacions.RequestMapping;
//import org.uoc.winter.anotacions.ModelAttribute;
import org.uoc.winter.anotacions.RequestParam;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.HashSet;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;

import org.apache.log4j.Logger;

import javassist.ClassClassPath;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtField;
import javassist.CtMethod;

import javax.servlet.ServletContext;

/*
 * Aquest codi esta basat en la informacio trobada sobre
 * el processador d'anotacions publicada a:
 * http://dyutiman.wordpress.com/tag/javassist/
 */
public class UtilJavassist
{
    private String classesPath = "/WEB-INF/classes/";
    private String libPath = "/WEB-INF/lib";
	private static final Logger log = Logger.getRootLogger();
	private ServletContext ctx;
	Set<String> allClasses = null;
	ClassPool pool = null;
		
	private void initclasspool()
	{
		String loghead="initclasspool> ";
		
		log.info(loghead+"begin");
		
	   	/*
         * Utilitzem javassist per recollir les classes. Inicialitzem 
         */
    	try {
           pool = ClassPool.getDefault();
    	   pool.insertClassPath(new ClassClassPath(UtilJavassist.class));
    	   pool.appendClassPath(libPath);
    	   
    	}
    	catch (Exception e) {
    		e.printStackTrace();
    	}
    	
    	log.info(loghead+"end");
	}
	
	// constructor basic que inicialitza classpool
	public UtilJavassist() {
		initclasspool();
	}
	
	// constructor que inicialitza classpool i obte la llista de classes que pertanyen al paquet o paquets
	public UtilJavassist(ServletContext ctx,String paquets,String sep) {
		this.ctx = ctx;
		
	    /*
	     * Creen un nou conjunt on posarem els fitxers de classes trobats a classesPath
	     */
    	allClasses = new HashSet<String>();
    	
    	/*
    	 * inicialitzem el conjunt amb el llistat de classes que ens interessen
    	 */
    	allClasses = BuscaClasses(paquets,allClasses,sep);
    	
    	initclasspool();
	}
	
    private Set<String> obtefitxersclasse(String basepath, String [] Paths, Set<String> finalList) {
    	String loghead="obtefitxersclasse> ";
    	
    	/*
    	 * Creen un conjunt amb els camins de recursos del contexte
    	 */
    	
    	log.info(loghead+"begin "+basepath);
    	
       Set<String> resourceSet = ctx.getResourcePaths(basepath);
    
       if (resourceSet == null) return finalList;
       
       for (Iterator<String> iterator = resourceSet.iterator(); iterator.hasNext();) {
          String resourcePath = (String) iterator.next();
          
    	  if (resourcePath.endsWith(".class")) {
    		  for (String Path : Paths) {
    		     int index=resourcePath.indexOf(Path);
 
    		     if (index!=-1) { // trobat
    		    	 
    		        String entrada = resourcePath.substring(index);
    		  
    		        /*
    		         * obtenim el nom de la classe real, preparat per ser utilitzat desde javassist
    		         */
    		        entrada = entrada.replace("/", ".").substring(0, entrada.indexOf(".class"));
    		     
    		        log.info(loghead+"AFEGIM: "+entrada);
    		        finalList.add(entrada);
    		     }
    		  }
    		  
    	  } else if (resourcePath.endsWith("/")) {
    		  log.info(loghead+"crida recursiva amb nou path="+resourcePath);
    	      obtefitxersclasse(resourcePath, Paths, finalList);
          }
       }
       
       log.info(loghead+"end trobats="+finalList.size());
       
       return finalList;
    }
    
    private Set<String> obtefitxersjars(String basepath,String [] Paths, Set<String> finalList) {
		String loghead="obtefitxersjars> ";
		
    	log.info(loghead+"begin basepath"+basepath);
    	
	    /*
	     * Prenem els directoris de recursos del contexte
	     */
	    Set<String> resourceSet = ctx.getResourcePaths(basepath);
	    if (resourceSet == null) return finalList;
	    
	    try {

	    	/*
	    	 * Extraiem les classes dels fitxers jars (comprimits amb zip)
	    	 */
	    	
	        for (Iterator<String> iterator = resourceSet.iterator(); iterator.hasNext();) {
	           String resourcePath = (String) iterator.next();
	    		   
	    	   JarInputStream jarInputStream = new JarInputStream(ctx.getResourceAsStream(resourcePath));
	    	   JarEntry jarEntry;
	    	
	    	   while ((jarEntry = jarInputStream.getNextJarEntry()) != null) {
	    	      String entrada = jarEntry.getName();
	    	      
	    	      for (String Path : Paths) {
	    	         if ((entrada.startsWith(Path)) && (jarEntry.getName().endsWith(".class"))) {
	                 
	     		        /*
	     		         * obtenim el nom de la classe real, preparat per ser utilitzat desde javassist
	     		         */
	                    entrada = entrada.replace("/", ".").substring(0, entrada.indexOf(".class"));
	                 
	                    log.info(loghead+"AFEGIM: "+entrada);
	    	            finalList.add(entrada);
	    	         }
	    	      }
	    	   }
	    	   
	    	   jarInputStream.close();
	    	}
        } catch (Exception e) {
           e.printStackTrace();
	    }
    	
    	log.info(loghead+"end trobats="+finalList.size());
    	
    	return finalList;
    }
    
	public Set<String> getClasses() { return allClasses; }
	
    /*
     * busquem el fitxer on es troba la classe o be les classes que pertanyen al paquet
     */
	public Set<String> BuscaClasses(String paquets,Set<String> allClasses,String sep) {
		String loghead="BuscaClasses> ";
		
		if (allClasses==null) {
			log.error(loghead+"allClasses = null");
			return allClasses;
		}
		
		log.info(loghead+"begin");
		
		String Path = paquets.replace(".", "/");
		String[] parts = Path.split(sep); // separador    	
    	
    	log.info(loghead+"Path="+Path);

 	    allClasses = obtefitxersclasse(classesPath, parts,  allClasses);
 	    allClasses = obtefitxersjars  (libPath    , parts,  allClasses);	
    	
    	log.info(loghead+"total="+allClasses.size());
    	log.info(loghead+"end");
    	
    	return allClasses;
	}
	
    /*
     * processem les anotacions de la llista de classes creada al constructor
     */
	public void ProcessarControllers(Mapes mapes) {
		String loghead="ProcessarControllers> ";
		
        if (allClasses==null) {
        	log.error(loghead+"allClasses = null");
        	return;
        }
        
	    try {
	    	
	    	/*
	    	 * llegim les anotacions (iterem per tots els fitxers de classes trobats
	    	 */
	    	for (String classFile : allClasses) {
	    	   Object [] annotations;
	    	   
               log.info(loghead+"considerem classe: "+classFile);
	    	   
               /*
	    	    * recuperem la classe
	    	    */
	    	   CtClass cc = pool.get(classFile);
	    	   CtMethod [] ctm = cc.getMethods();
	    	   
	    	   /*
	    	    * recuperem les anotacions
	    	    */
	    	   annotations = cc.getAnnotations();
	    	   if (annotations.length==0) continue;
	    	   
	    	   log.info(loghead+annotations.length+" anotacions trobades a la classe");
	    	   
	    	   for (Object annotation : annotations) {
	    	      log.info(loghead+"anotacio="+annotation.toString());
	    	      if (annotation instanceof Controller) {
	    	         String root="/";
	    	            
	    	     	 Controller tag = (Controller) annotation;
	    	         
	    	         log.info(loghead+"value: " + tag.value());
	    	         
	    	         if (root.equals(tag.value())==false) root = tag.value();
	    	         
	    	         // eliminem el darrer "/"    
	    	         if (root.endsWith("/") ) root = root.substring(0,root.length()-1);
	    	         
	    	         // si no hi ha metodes per explorar, continuem a la seguent classe
	    		     if  (ctm.length==0) continue;
	    		    	   
	    			 log.info(loghead+ctm.length+" metodes trobats a la classe");
	    			    	   
	    			 for (CtMethod cm : ctm) {
	    
	    				// anotacions del metode
	    			    annotations = cm.getAnnotations();
	    			    if (annotations.length==0) continue;
	    			       	   
	    			    log.info(loghead+annotations.length+" anotacions trobades al metode");
	    				for (Object annotation2 : annotations) {
	    				   log.info(loghead+"anotacio="+annotation2.toString());
	    				    		   
	    				   if (annotation2 instanceof RequestMapping) {
	    					   RequestMapping tag2 = (RequestMapping) annotation2;
	    					   String cami;
	    				    	         
	    					  Mapa mapa = new Mapa();
	    					   
	    				  	  log.info(loghead+"value: '" + tag2.value()+"' method:"+tag2.method());
	    				      
	    				  	  if (tag2.value().isEmpty()) {
	    				  		  /* si no hem passat valor per parametre agafem el nom del metode */
	    				  		  cami = cm.getName();
	    				  	  }
	    				  	  else {
	    				  		  cami = tag2.value();
	    				  	  }
	    				  	  
	    				  	  // eliminem el primer "/"
	    				  	  if (cami.startsWith("/")) cami = cami.substring(1);
	    				   
	    				  	  mapa.set_classe(cc.getName());
	    				      mapa.set_metode(cm.getName());
	    				      mapa.set_cami(root + "/" + cami);
	    				      mapa.set_httpmetode(tag2.method());  
	    				      
	    				      log.info(loghead+"afegim nou mapa");
	    				      mapes.add(mapa);
	    				   }
	    		    	}
	    				
	    				// anotacions dels parametres
	    			    annotations = cm.getParameterAnnotations();
	    			    if (annotations.length==0) continue;
	    			       	   
	    			    log.info(loghead+annotations.length+" anotacions trobades als parametres");
	    				for (Object annotation2 : annotations) {
	    				   log.info(loghead+"anotacio="+annotation2.toString());
	    				    		   
	    				   if (annotation2 instanceof RequestParam) {
	    					  RequestParam tag2 = (RequestParam) annotation2; 
	    				    	         
	    				  	  log.info(loghead+"value: '" + tag2.value());
	    				      
	    				  	  //if (cami.equals(tag2.value())==false) cami += tag2.value();
	    				    	 /*
	    				  	  
	    				  	  mapa.add_parameter(tag2.value());
	    				      mapa.set_cami(cami);
	    				      mapa.set_metode(tag2.method());
	    				    	   */           
	    				      log.info(loghead+"afegim nou parametre");
	    				    //  mapes.add(mapa);
	    				   }
	    		    	}
	    	         }
	    	      }
	    	   }
	    	}	    	
	    } catch (Exception e) {
           e.printStackTrace();
	    }

	    log.info(loghead+"end");
	}
	
    /*
     * processem les anotacions de configuracio de un nom de classe pasada per parametre 
     * (fully-qualified=amb el nom de paquet inclos)
     */
	public void ProcessarConfiguracions(String [] classes,HashMap<String,String> params) {
		String loghead="ProcessarConfiguracions> ";
        
    	log.info(loghead+"begin");

    	/*
    	 * Processem un a un totes les classes pasades per parametre, en forma de array
    	 */
        
	    try {
			
	    	/*
	    	 * llegim les anotacions (iterem per tots els fitxers de classes trobats
	    	 */
	    	for (String part : classes) {
	    	   Object [] annotations;
	    	   
               /*
	    	    * recuperem la classe
	    	    */
	    	   CtClass cc = pool.get(part);
	    	   
	    	   /*
	    	    * recuperem els camps de la classe
	    	    */
	    	   CtField [] cfs = cc.getFields();
	    	   log.info(loghead+"considerem classe: "+part+": "+cfs.length+" camps trobats");
	    	   
	    	   for (CtField cf : cfs) {
	    		   
	    		   annotations = cf.getAnnotations();
	    		   
		    	   log.info(loghead+"camp="+cf.toString()+": "+annotations.length+" anotacions");
		    	   
		    	   for (Object annotation : annotations) {
		    		   
		    	      if (annotation instanceof ContextAttr) {
		    	         ContextAttr anotacio = (ContextAttr) annotation;
		    	         
		    	         String tag = anotacio.tag(); 
		    	         log.info(loghead+"anotacio="+anotacio.toString());
		    	             
		    	         /*
		    	          * retornem el valor del tag per poder-lo convertir en atribut accessible al DispatcherServlet
		    	          */
		    	         String v = (String)cf.getConstantValue();
		    	         if (v!=null) {
		    	            log.info(loghead+"parametre "+tag+"="+v);
		    	            
		    	            /*
		    	             * identifiquem el key amb group@tag
		    	             */
	    	        	   	params.put(anotacio.group().toUpperCase()+"@"+tag.toUpperCase(), v);
		    	         }
		    	      }
		    	   }
	    	   }
	    	   
	    	}	    	
	    } catch (Exception e) {
           e.printStackTrace();
	    }

	    log.info(loghead+"end");
	}
}